/**
 * Copyright (c) 2015-2016 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.objectweb.petals.tools.rmi.server.remote.implementations;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Collections;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.ConcurrentHashMap;
import java.util.concurrent.ConcurrentMap;
import java.util.logging.Logger;

import javax.jbi.JBIException;
import javax.jbi.component.ComponentContext;
import javax.jbi.messaging.DeliveryChannel;
import javax.jbi.messaging.MessageExchange;

import org.objectweb.petals.tools.rmi.remote.RemoteComponentContextClient;
import org.objectweb.petals.tools.rmi.remote.RemoteComponentContextClientFactory;
import org.ow2.petals.component.framework.util.ServiceEndpointKey;
import org.w3c.dom.Document;

/**
 * The RMI implementation of component context client.
 * 
 * @author Christophe DENEUX - Linagora
 */
public class RemoteComponentContextClientFactoryImpl extends UnicastRemoteObject implements
        RemoteComponentContextClientFactory {

    private static final long serialVersionUID = -1266197929011717656L;

    private final SharedComponentData sharedData;

    public RemoteComponentContextClientFactoryImpl(final SharedComponentData sharedData) throws RemoteException {
        this.sharedData = sharedData;
    }

    @Override
    public RemoteComponentContextClient createRemoteComponentContextClient() throws RemoteException {

        try {
            return new RemoteComponentContextClientImpl(this.sharedData);
        } catch (final JBIException e) {
            throw new RemoteException("An error occurs creating the component context client 'consumer'.", e);
        }
    }

    public static class SharedComponentData {
        public final ComponentContext componentContext;

        public final DeliveryChannel deliveryChannel;

        /**
         * HasMap containing all the messages exchanges.
         */
        public final ConcurrentMap<String, MessageExchange> messages = new ConcurrentHashMap<>();

        /**
         * Message exchanges received by the JBI listener, waiting that a delivery channel accept them. One
         * {@link BlockingQueue} per RMI clients.
         */
        public final ConcurrentMap<String, BlockingQueue<MessageExchange>> allReceivedMessages = new ConcurrentHashMap<>();

        /**
         * mapping between the endpoints registered on our component and their description.
         */
        public final ConcurrentMap<ServiceEndpointKey, Document> ownedEndpointDescriptor = new ConcurrentHashMap<>();

        /**
         * Mapping between the endpoints registered on our component and the provider ids in the
         * {@link #allReceivedMessages} map.
         */
        public final ConcurrentMap<ServiceEndpointKey, String> ownedEndpointProviders = new ConcurrentHashMap<>();

        /**
         * The threads receiving message through the JBI listeners, waiting that the remote delivery channel accept
         * message exchange for them.
         */
        public final Collection<Thread> threadReceivingMessages = Collections.synchronizedList(new ArrayList<Thread>());

        public final Logger logger;

        public SharedComponentData(final ComponentContext componentContext, final DeliveryChannel deliveryChannel,
                final Logger logger) {
            this.componentContext = componentContext;
            this.deliveryChannel = deliveryChannel;
            this.logger = logger;

        }

    }
}
